## 9.1

library('httr')
library('rvest')

response <- GET('http://codingsocialscience.org/')

website_content <- content( response , 'text' )
parsed_website <- read_html( website_content )

for ( node in html_nodes( parsed_website , 'li' ) ) {
        print( html_text(node) )
}

## 9.2

library('httr')
library('rvest')

response <- GET('https://uk.sagepub.com/')

website_content <- content( response , 'text' )
parsed_website <- read_html( website_content )

image_count <- 0

for ( node in html_nodes( parsed_website , 'img' ) ) {
    image_count <- image_count + 1
}
    
print( image_count )

## 9.3

library('httr')
library('rvest')

response <- GET('https://www.helsinki.fi/')

website_content <- content( response , 'text' )
website_content <- tolower( website_content )

search_words = list( 'Twitter', 'Facebook', 'YouTube' )

for( search_word in search_words ) {
    if( grepl( tolower(search_word), website_content ) ) {
        print( paste( "Found", search_word) )
    }
}

## 9.4

library('httr')
library('rvest')
library('stringr')

universities <- list('https://www.helsinki.fi', 'https://www.aalto.fi')
results = list()

search_words = list( 'Twitter', 'Facebook', 'YouTube' )

for( university in universities ){
    
    response <- GET(university)

    website_content <- content( response , 'text' )
    website_content <- tolower( website_content )
    
    res <- list()
    
    for( search_word in search_words ) {
        matches <- str_count( website_content, tolower(search_word) )
        res[[ search_word ]] <- matches
    }
    
    results[[ university ]] <- res
    
} 

print( results )

## 9.5

if( FALSE ) { ## this code could not be automatically tested, so comment out

library('httr')
library('rvest')


start_url <- "http://codingsocialscience.org/"

start_page <- GET(start_url)
start_page <- read_html( start_page )


for( link in html_nodes( start_page , 'a' ) ) {
    link <- html_attr( link , 'href')
    
    if( startsWith( link, 'http' ) ) {
        
        followup <- GET( link )
        print( link )
        followup <- read_html( followup )
        
        for( followuplink in html_nodes( followup, 'a' ) ) {
            followuplink <- html_attr( followuplink, 'href' )
            ## some site does not define href on their link, this takes care of that issue
            ## followuplink <- ifelse( is.na( followuplink ) , '' , followuplink )
            if( startsWith( followuplink  , 'http') ) {
               print( paste( link, '-', followuplink ) )
            }
        }
    }
}
    
}

## 9.6

library('httr')
library('jsonlite')

url <- 'https://data.police.uk/api/crimes-street/all-crime?lat=51.5073&lng=-0.171505' ## latest month is shown by default, see documentation

data <- GET(url)
data <- content( data, 'text' )
data <- fromJSON( data ) ## could just be URL, but we do a bit step-by-step here

categories = list() ## this is a megacollector

for( rn in 1:nrow(data) ) {
    
    category <- data[rn, 'category']
    
    if( ! category %in% names( categories) ) {
        categories[[ category ]] <- 0
    }
    
    categories[[ category ]] <- categories[[ category ]] + 1

}

print( categories )

## 9.7

library('httr')
library('jsonlite')

url <- 'http://api.worldbank.org/v2/country/fi?format=json' ## latest month is shown by default, see documentation

data <- GET(url)
data <- content( data, 'text' )
data <- fromJSON( data ) ## could just be URL, but we do a bit step-by-step here

print( data[[2]][1, 'capitalCity'] )

## 9.8

library('httr')
library('jsonlite')

adjacents <- list('se', 'no', 'ru', 'ee')

for( country in adjacents ) {
    url <- paste( 'http://api.worldbank.org/v2/country/', country, '?format=json' , sep = '' )
    
    data <- GET(url)
    data <- content( data, 'text' )
    data <- fromJSON( data ) ## could just be URL, but we do a bit step-by-step here

    print( data[[2]][1, 'capitalCity'] )
}

## 9.9

library('httr')
library('jsonlite')

url <- 'http://api.worldbank.org/v2/country/all?format=json'

data <- GET(url)
data <- content( data, 'text' )
data <- fromJSON( data ) ## could just be URL, but we do a bit step-by-step here

## the book does not cover pagination, so let's this is not a full result

data = data[[2]] ## index 1 is for pagination details

for( rn in 1:nrow(data) ) {
    print( paste( data[rn, 'latitude'], data[rn, 'longitude'], data[rn, 'incomeLevel']['value'] ) )
}

## 9.10

example <- '127.0.0.1 - - [10/Nov/2020:08:43:18 +0200] "GET / HTTP/1.1" 200 4471 "-" "Mozilla/5.0"'
example <- strsplit(example, '\\[' )[[1]][2] ## everything to right from [
example <- strsplit(example, '\\]' )[[1]][1]  ## everything to left from ]
## 10/Nov/2020:08:43:18 +0200 # you can print it out here
## print( example )
example <- strsplit( example, ':')[[1]][1] ## everything left from :
date <- strsplit( example, '/')[[1]]
day <- date[1]
month <- date[2]
year <- date[3]


print( paste( year, month, day ) )

## 9.11

library('readtext')

all_text <- readtext('alice.pdf')
all_text <- all_text[['text']]

print( all_text )

## 9.12

library('readtext')

all_text <- readtext('alice.docx')
all_text <- all_text[['text']]

print( all_text )

## 9.13

library('httr')
library('rvest')
library('stringr')
library('jsonlite')

universities <- list('https://www.helsinki.fi', 'https://www.aalto.fi')
results = list()

search_words = list( 'Twitter', 'Facebook', 'YouTube' )

for( university in universities ){
    
    response <- GET(university)

    website_content <- content( response , 'text' )
    website_content <- tolower( website_content )
    
    res <- list()
    
    for( search_word in search_words ) {
        matches <- str_count( website_content, tolower(search_word) )
        res[[ search_word ]] <- matches
    }
    
    results[[ university ]] <- res
    
}

write( toJSON( results, auto_unbox = TRUE ), file = 'universities.json' ) ## auto_unbox ensures that the single values are not stored as lists, more common way of presenting these in JSON

## 9.14

library('jsonlite')

data <- read_json("universities.json")

for( university in names( data ) ) {
    results <- data[[university]]
    counts <- 0
    for( count in unlist( results ) ) {
        counts <- counts + count
    }
    print( paste( university, counts ) )
}



## 9.15

library('jsonlite')

## this could also be a dictionary in dictionary
northwest <- list()
northeast <- list()
southwest <- list()
southeast <- list()

data <- read_json( "countries.json" )

for( country in data ) {
    income <- country[['income']]
    if( country[['lat']] > 0 ) {
        if( country[['long']] > 0 ) {
            if( ! income %in% names( northeast ) ) {
                northeast[[income]] <- 0
            }
            northeast[[income]] <- northeast[[income]] + 1
        } else {
            if( ! income %in% names( northwest ) ) {
                northwest[[income]] <- 0
            }
            northwest[[income]] <- northwest[[income]] + 1
        }
    } else {
        if( country[['long']] > 0 ) {
            if( ! income %in% names( southeast ) ) {
                southeast[[income]] <- 0
            }
            southeast[[income]] <- southeast[[income]] + 1
        } else {
            if( ! income %in% names( southwest ) ) {
                southwest[[income]] <- 0
            }
            southwest[[income]] <- southwest[[income]] + 1
        }
    }
}

print( paste( northwest, northeast, southwest, southeast ) )

## 9.16

library('jsonlite')

url = 'https://data.police.uk/api/crimes-street/all-crime?lat=51.5073&lng=-0.171505'

dataframe <- fromJSON( url )

print( table( dataframe$category ) )
